# Django Turbo Response
from turbo_response import Action, render_turbo_frame, render_turbo_stream
from turbo_response.renderers import Jinja2


class TestRenderTurboStream:
    def test_render_empty_stream(self):
        s = render_turbo_stream(action=Action.REMOVE, target="test")
        assert (
            s
            == '<turbo-stream action="remove" target="test"><template></template></turbo-stream>'
        )

    def test_render_content(self):
        s = render_turbo_stream(
            action=Action.REPLACE,
            target="test",
            content="my content",
        )
        assert (
            s
            == '<turbo-stream action="replace" target="test"><template>my content</template></turbo-stream>'
        )

    def test_render_content_with_jinja2(self):
        s = render_turbo_stream(
            action=Action.REPLACE,
            target="test",
            content="my content",
            renderer=Jinja2(),
        )
        assert (
            s
            == '<turbo-stream action="replace" target="test"><template>my content</template></turbo-stream>'
        )

    def test_render_content_xss(self):
        s = render_turbo_stream(
            action=Action.REPLACE,
            target="test",
            content="<script></script>",
        )
        assert (
            s
            == '<turbo-stream action="replace" target="test"><template>&lt;script&gt;&lt;/script&gt;</template></turbo-stream>'
        )

    def test_render_content_jinja2_xss(self):
        s = render_turbo_stream(
            action=Action.REPLACE,
            target="test",
            content="<script></script>",
            renderer=Jinja2(),
        )
        assert (
            s
            == '<turbo-stream action="replace" target="test"><template>&lt;script&gt;&lt;/script&gt;</template></turbo-stream>'
        )

    def test_render_content_is_safe(self):
        s = render_turbo_stream(
            action=Action.REPLACE,
            target="test",
            content="<script></script>",
            is_safe=True,
        )
        assert (
            s
            == '<turbo-stream action="replace" target="test"><template><script></script></template></turbo-stream>'
        )

    def test_render_content_is_safe_jinja2(self):
        s = render_turbo_stream(
            action=Action.REPLACE,
            target="test",
            content="<script></script>",
            is_safe=True,
            renderer=Jinja2(),
        )
        assert (
            s
            == '<turbo-stream action="replace" target="test"><template><script></script></template></turbo-stream>'
        )


class TestRenderTurboFrame:
    def test_render_empty_frame(self):
        s = render_turbo_frame("test")
        assert s == '<turbo-frame id="test"></turbo-frame>'

    def test_render_content(self):
        s = render_turbo_frame(
            dom_id="test",
            content="my content",
        )
        assert s == '<turbo-frame id="test">my content</turbo-frame>'

    def test_render_xss(self):
        s = render_turbo_frame(
            dom_id="test",
            content="<script></script>",
        )
        assert s == '<turbo-frame id="test">&lt;script&gt;&lt;/script&gt;</turbo-frame>'

    def test_render_xss_jinja2(self):
        s = render_turbo_frame(
            dom_id="test", content="<script></script>", renderer=Jinja2()
        )
        assert s == '<turbo-frame id="test">&lt;script&gt;&lt;/script&gt;</turbo-frame>'

    def test_render_content_jinja2(self):
        s = render_turbo_frame(dom_id="test", content="my content", renderer=Jinja2())
        assert s == '<turbo-frame id="test">my content</turbo-frame>'

    def test_render_content_is_safe(self):
        s = render_turbo_frame(
            dom_id="test",
            content="<script></script>",
            is_safe=True,
        )
        assert s == '<turbo-frame id="test"><script></script></turbo-frame>'

    def test_render_content_is_safe_jinja2(self):
        s = render_turbo_frame(
            dom_id="test", content="<script></script>", is_safe=True, renderer=Jinja2()
        )
        assert s == '<turbo-frame id="test"><script></script></turbo-frame>'



# Standard Library
import http

# Django
from django import forms
from django.views.generic import CreateView

# Third Party Libraries
import pytest

# Django Turbo Response
from turbo_response import Action
from turbo_response.mixins import TurboFormAdapterMixin
from turbo_response.tests.testapp.forms import TodoForm
from turbo_response.tests.testapp.models import TodoItem
from turbo_response.views import (
    TurboCreateView,
    TurboFormView,
    TurboFrameTemplateView,
    TurboFrameView,
    TurboStreamCreateView,
    TurboStreamDeleteView,
    TurboStreamFormView,
    TurboStreamTemplateView,
    TurboStreamUpdateView,
    TurboStreamView,
    TurboUpdateView,
)

pytestmark = pytest.mark.django_db


class MyForm(forms.Form):
    description = forms.CharField()


class TestTurboFormAdapterView:
    """Adapt an existing view without touching internals"""

    class MyView(TurboFormAdapterMixin, CreateView):
        form_class = TodoForm
        model = TodoItem
        success_url = "/done/"

    def test_get(self, rf):
        req = rf.get("/")
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert "form" in resp.context_data
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_get_with_explicit_target(self, rf):
        req = rf.get("/")
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert "form" in resp.context_data
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_post_with_validation_errors(self, rf):
        req = rf.post("/", {})
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.UNPROCESSABLE_ENTITY
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_post_success(self, rf):
        req = rf.post("/", {"description": "ok"})
        resp = self.MyView.as_view()(req)
        assert resp.url == "/done/"
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        assert TodoItem.objects.count() == 1


class TestTurboStreamView:
    def test_get(self, rf):
        class MyView(TurboStreamView):
            def get_response_content(self):
                return "hello"

        req = rf.get("/")
        resp = MyView.as_view(
            turbo_stream_target="test", turbo_stream_action=Action.REPLACE
        )(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert "text/vnd.turbo-stream.html;" in resp["Content-Type"]
        assert resp.content.startswith(
            b'<turbo-stream action="replace" target="test"><template>hello'
        )

    def test_get_xss(self, rf):
        class MyView(TurboStreamView):
            def get_response_content(self):
                return "<script />"

        req = rf.get("/")
        resp = MyView.as_view(
            turbo_stream_target="test", turbo_stream_action=Action.REPLACE
        )(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert "text/vnd.turbo-stream.html;" in resp["Content-Type"]
        assert resp.content.startswith(
            b'<turbo-stream action="replace" target="test"><template>&lt;script /&gt;'
        )

    def test_get_is_safe(self, rf):
        class MyView(TurboStreamView):
            def get_response_content(self):
                return "<script />"

            def render_turbo_stream(self, **kwargs):
                return super().render_turbo_stream(is_safe=True, **kwargs)

        req = rf.get("/")
        resp = MyView.as_view(
            turbo_stream_target="test", turbo_stream_action=Action.REPLACE
        )(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert "text/vnd.turbo-stream.html;" in resp["Content-Type"]
        assert resp.content.startswith(
            b'<turbo-stream action="replace" target="test"><template><script />'
        )


class TestTurboStreamTemplateView:
    def test_get(self, rf):
        class MyView(TurboStreamTemplateView):
            template_name = "simple.html"

            def get_context_data(self, **kwargs):
                return {**kwargs, "msg": "my content"}

        req = rf.get("/")
        resp = MyView.as_view(
            turbo_stream_target="test", turbo_stream_action=Action.REPLACE
        )(req)

        assert resp.status_code == http.HTTPStatus.OK
        assert "text/vnd.turbo-stream.html;" in resp["Content-Type"]
        assert "is_turbo_stream" in resp.context_data
        assert resp.template_name == ["simple.html"]
        assert resp.render().content.startswith(
            b'<turbo-stream action="replace" target="test"><template><div>my content'
        )


class TestTurboCreateView:
    class MyView(TurboCreateView):
        form_class = TodoForm
        model = TodoItem
        success_url = "/done/"

    def test_get(self, rf):
        req = rf.get("/")
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert "form" in resp.context_data
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_get_with_explicit_target(self, rf):
        req = rf.get("/")
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert "form" in resp.context_data
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_post_with_validation_errors(self, rf):
        req = rf.post("/", {})
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.UNPROCESSABLE_ENTITY
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_post_success(self, rf):
        req = rf.post("/", {"description": "ok"})
        resp = self.MyView.as_view()(req)
        assert resp.url == "/done/"
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        assert TodoItem.objects.count() == 1


class TestTurboStreamCreateView:
    class MyView(TurboStreamCreateView):
        form_class = TodoForm
        model = TodoItem
        success_url = "/done/"

    def test_get(self, rf):
        req = rf.get("/")
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert "is_turbo_stream" not in resp.context_data
        assert "form" in resp.context_data
        assert resp.context_data["turbo_stream_target"] == "form-todoitem"
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_post_with_validation_errors(self, rf):
        req = rf.post("/", {})
        resp = self.MyView.as_view()(req)
        assert "is_turbo_stream" in resp.context_data
        assert "form" in resp.context_data
        assert resp.context_data["turbo_stream_target"] == "form-todoitem"
        assert resp.template_name == ["testapp/_todoitem_form.html"]

    def test_post_success(self, rf):
        req = rf.post("/", {"description": "ok"})
        resp = self.MyView.as_view()(req)
        assert resp.url == "/done/"
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        assert TodoItem.objects.count() == 1


class TestTurboUpdateView:
    class MyView(TurboUpdateView):
        form_class = TodoForm
        model = TodoItem
        success_url = "/done/"

    def test_get(self, rf, todo):
        req = rf.get("/")
        resp = self.MyView.as_view()(req, pk=todo.pk)
        assert resp.status_code == http.HTTPStatus.OK
        assert "form" in resp.context_data
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_post_with_validation_errors(self, rf, todo):
        req = rf.post("/", {})
        resp = self.MyView.as_view()(req, pk=todo.pk)
        assert resp.status_code == http.HTTPStatus.UNPROCESSABLE_ENTITY
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_post_success(self, rf, todo):
        req = rf.post("/", {"description": "updated!"})
        resp = self.MyView.as_view()(req, pk=todo.pk)
        assert resp.url == "/done/"
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        todo.refresh_from_db()
        assert todo.description == "updated!"


class TestTurboStreamUpdateView:
    class MyView(TurboStreamUpdateView):
        form_class = TodoForm
        model = TodoItem
        success_url = "/done/"

    def test_get(self, rf, todo):
        req = rf.get("/")
        resp = self.MyView.as_view()(req, pk=todo.pk)
        assert resp.status_code == http.HTTPStatus.OK
        assert "is_turbo_stream" not in resp.context_data
        assert "form" in resp.context_data
        assert resp.context_data["turbo_stream_target"] == f"form-todoitem-{todo.pk}"
        assert resp.template_name == ["testapp/todoitem_form.html"]

    def test_post_with_validation_errors(self, rf, todo):
        req = rf.post("/", {})
        resp = self.MyView.as_view()(req, pk=todo.pk)
        assert "is_turbo_stream" in resp.context_data
        assert resp.context_data["turbo_stream_target"] == f"form-todoitem-{todo.pk}"
        assert resp.template_name == ["testapp/_todoitem_form.html"]

    def test_post_success(self, rf, todo):
        req = rf.post("/", {"description": "updated!"})
        resp = self.MyView.as_view()(req, pk=todo.pk)
        assert resp.url == "/done/"
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        todo.refresh_from_db()
        assert todo.description == "updated!"


class TestTurboFormView:
    class MyView(TurboFormView):
        form_class = MyForm
        template_name = "my_form.html"
        success_url = "/done/"

    def test_get(self, rf):
        req = rf.get("/")
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert "form" in resp.context_data
        assert resp.template_name == ["my_form.html"]

    def test_post_with_validation_errors(self, rf):
        req = rf.post("/", {})
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.UNPROCESSABLE_ENTITY
        assert resp.template_name == ["my_form.html"]

    def test_post_success(self, rf):
        req = rf.post("/", {"description": "ok"})
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        assert resp.url == "/done/"


class TestTurboStreamFormView:
    class MyView(TurboStreamFormView):
        form_class = MyForm
        template_name = "my_form.html"
        success_url = "/done/"
        turbo_stream_target = "my-form"

    def test_get(self, rf):
        req = rf.get("/")
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert "form" in resp.context_data
        assert "is_turbo_stream" not in resp.context_data
        assert resp.context_data["turbo_stream_target"] == "my-form"
        assert resp.template_name == ["my_form.html"]

    def test_post_with_validation_errors(self, rf):
        req = rf.post("/", {})
        resp = self.MyView.as_view()(req)
        assert "is_turbo_stream" in resp.context_data
        assert resp.context_data["turbo_stream_target"] == "my-form"
        assert resp.template_name == ["_my_form.html"]

    def test_post_success(self, rf):
        req = rf.post("/", {"description": "ok"})
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        assert resp.url == "/done/"


class TestTurboStreamDeleteView:
    class MyView(TurboStreamDeleteView):
        template_name = "simple.html"
        model = TodoItem

    def test_post(self, rf, todo):
        req = rf.post("/")
        resp = self.MyView.as_view()(req, pk=todo.pk)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/vnd.turbo-stream.html; charset=utf-8"
        assert f'target="todoitem-{todo.pk}"' in str(resp.content)
        assert TodoItem.objects.count() == 0


class TestTurboFrameView:
    def test_get(self, rf):
        class MyView(TurboFrameView):
            turbo_frame_dom_id = "test"

            def get_response_content(self):
                return "done"

        req = rf.get("/")
        resp = MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert resp.content == b'<turbo-frame id="test">done</turbo-frame>'

    def test_get_xss(self, rf):
        class MyView(TurboFrameView):
            turbo_frame_dom_id = "test"

            def get_response_content(self):
                return "<script />"

        req = rf.get("/")
        resp = MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert resp.content == b'<turbo-frame id="test">&lt;script /&gt;</turbo-frame>'

    def test_get_is_safe(self, rf):
        class MyView(TurboFrameView):
            turbo_frame_dom_id = "test"

            def get_response_content(self):
                return "<script />"

            def render_turbo_frame(self):
                return super().render_turbo_frame(is_safe=True)

        req = rf.get("/")
        resp = MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert resp.content == b'<turbo-frame id="test"><script /></turbo-frame>'


class TestTurboFrameTemplateView:
    class MyView(TurboFrameTemplateView):
        turbo_frame_dom_id = "test"
        template_name = "simple.html"

        def get_context_data(self, **kwargs):
            return {**kwargs, "msg": "my content"}

    def test_get(self, rf):
        req = rf.get("/")
        resp = self.MyView.as_view()(req)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert resp.render().content.startswith(
            b'<turbo-frame id="test"><div>my content</div>'
        )



# Django Turbo Response
from turbo_response import TurboFrame


class TestTurboFrame:
    def test_render(self):
        s = TurboFrame("my-form").render("OK")
        assert s == '<turbo-frame id="my-form">OK</turbo-frame>'

    def test_render_xss(self):
        s = TurboFrame("my-form").render("<script></script>")
        assert (
            s == '<turbo-frame id="my-form">&lt;script&gt;&lt;/script&gt;</turbo-frame>'
        )

    def test_render_is_safe(self):
        s = TurboFrame("my-form").render("<script></script>", is_safe=True)
        assert s == '<turbo-frame id="my-form"><script></script></turbo-frame>'

    def test_template(self):
        s = (
            TurboFrame("my-form")
            .template("simple.html", {"msg": "my content"})
            .render()
        )
        assert "my content" in s
        assert '<turbo-frame id="my-form">' in s

    def test_response(self):
        resp = TurboFrame("my-form").response("OK")
        assert resp.status_code == 200
        assert b"OK" in resp.content
        assert b'<turbo-frame id="my-form"' in resp.content

    def test_response_xss(self):
        resp = TurboFrame("my-form").response("<script></script>")
        assert resp.status_code == 200
        assert b"&lt;script&gt;&lt;/script&gt;" in resp.content
        assert b'<turbo-frame id="my-form"' in resp.content

    def test_response_is_safe(self):
        resp = TurboFrame("my-form").response("<script></script>", is_safe=True)
        assert resp.status_code == 200
        assert b"<script></script>" in resp.content
        assert b'<turbo-frame id="my-form"' in resp.content

    def test_template_render(self, rf):
        req = rf.get("/")
        s = (
            TurboFrame("my-form")
            .template("simple.html", {"msg": "my content"}, request=req)
            .render()
        )
        assert "my content" in s
        assert '<turbo-frame id="my-form"' in s

    def test_template_render_xss(self, rf):
        req = rf.get("/")
        s = (
            TurboFrame("my-form")
            .template("simple.html", {"msg": "<script></script>"}, request=req)
            .render()
        )
        assert "&lt;script&gt;&lt;/script&gt;" in s
        assert '<turbo-frame id="my-form"' in s

    def test_template_render_req_in_arg(self, rf):
        req = rf.get("/")
        s = (
            TurboFrame("my-form")
            .template("simple.html", {"msg": "my content"})
            .render(request=req)
        )
        assert "my content" in s
        assert '<turbo-frame id="my-form"' in s

    def test_template_response(self, rf):
        req = rf.get("/")
        resp = (
            TurboFrame("my-form")
            .template("simple.html", {"msg": "my content"}, request=req)
            .response()
        )
        assert resp.status_code == 200
        assert "is_turbo_frame" in resp.context_data
        assert resp._request == req
        content = resp.render().content
        assert b"my content" in content
        assert b'<turbo-frame id="my-form"' in content

    def test_template_response_req_in_arg(self, rf):
        req = rf.get("/")
        resp = (
            TurboFrame("my-form")
            .template("simple.html", {"msg": "my content"})
            .response(req)
        )
        assert resp.status_code == 200
        assert "is_turbo_frame" in resp.context_data
        assert resp._request == req
        content = resp.render().content
        assert b"<div>my content</div>" in content
        assert b'<turbo-frame id="my-form"' in content



# Standard Library
import pathlib

SECRET_KEY = "o7pb3_ibyl@3%jhc85qj*dgh5etxmzxpg89%$85$#v92&8e0ub"
DATABASES = {
    "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": "mem_db"},
}
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [pathlib.Path(__file__).parent.absolute() / "templates"],
        "APP_DIRS": True,
        "OPTIONS": {
            "debug": False,
            "context_processors": [],
            "builtins": [],
            "libraries": {},
        },
    }
]
INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "turbo_response",
    "turbo_response.tests.testapp.apps.TestAppConfig",
]

ROOT_URLCONF = "turbo_response.tests.testapp.urls"

TURBO_RESPONSE_RENDERER = "turbo_response.renderers.TemplatesSetting"



# Django Turbo Response
from turbo_response import TurboStream


class TestTurboStream:
    def test_render(self):
        s = TurboStream("my-form").append.render("OK").strip()
        assert (
            s
            == '<turbo-stream action="append" target="my-form"><template>OK</template></turbo-stream>'
        )

    def test_render_xss(self):
        s = TurboStream("my-form").append.render("<script></script>").strip()
        assert (
            s
            == '<turbo-stream action="append" target="my-form"><template>&lt;script&gt;&lt;/script&gt;</template></turbo-stream>'
        )

    def test_render_is_safe(self):
        s = (
            TurboStream("my-form")
            .append.render("<script></script>", is_safe=True)
            .strip()
        )
        assert (
            s
            == '<turbo-stream action="append" target="my-form"><template><script></script></template></turbo-stream>'
        )

    def test_template(self):
        s = (
            TurboStream("my-form")
            .append.template("simple.html", {"msg": "my content"})
            .render()
        )
        assert "my content" in s
        assert '<turbo-stream action="append" target="my-form">' in s

    def test_template_xss(self):
        s = (
            TurboStream("my-form")
            .append.template("simple.html", {"msg": "<script></script>"})
            .render()
        )
        assert "&lt;script&gt;&lt;/script&gt;" in s
        assert '<turbo-stream action="append" target="my-form">' in s

    def test_template_with_req_init(self, rf):
        s = (
            TurboStream("my-form")
            .append.template("simple.html", {"msg": "my content"}, request=rf.get("/"))
            .render()
        )
        assert "my content" in s
        assert '<turbo-stream action="append" target="my-form">' in s

    def test_template_with_req_arg(self, rf):
        s = (
            TurboStream("my-form")
            .append.template("simple.html", {"msg": "my content"})
            .render(request=rf.get("/"))
        )
        assert "my content" in s
        assert '<turbo-stream action="append" target="my-form">' in s

    def test_response(self):
        resp = TurboStream("my-form").append.response("OK")
        assert resp.status_code == 200
        assert b"OK" in resp.content
        assert b'<turbo-stream action="append" target="my-form"' in resp.content

    def test_response_xss(self):
        resp = TurboStream("my-form").append.response("<script></script>")
        assert resp.status_code == 200
        assert b"&lt;script&gt;&lt;/script&gt;" in resp.content
        assert b'<turbo-stream action="append" target="my-form"' in resp.content

    def test_response_is_safe(self):
        resp = TurboStream("my-form").append.response("<script></script>", is_safe=True)
        assert resp.status_code == 200
        assert b"<script></script>" in resp.content
        assert b'<turbo-stream action="append" target="my-form"' in resp.content

    def test_template_response_req_init(self, rf):
        req = rf.get("/")
        resp = (
            TurboStream("my-form")
            .append.template("simple.html", {"msg": "my content"}, request=req)
            .response()
        )
        assert resp.status_code == 200
        assert "is_turbo_stream" in resp.context_data
        assert resp._request == req
        content = resp.render().content
        assert b"my content" in content
        assert b'<turbo-stream action="append" target="my-form"' in content

    def test_template_response(self, rf):
        req = rf.get("/")
        resp = (
            TurboStream("my-form")
            .append.template("simple.html", {"msg": "my content"})
            .response(req)
        )
        assert resp.status_code == 200
        assert "is_turbo_stream" in resp.context_data
        assert resp._request == req
        content = resp.render().content
        assert b"<div>my content</div>" in content
        assert b'<turbo-stream action="append" target="my-form"' in content



import pathlib
from functools import lru_cache
from typing import Optional

from django.forms.renderers import BaseRenderer
from django.forms.renderers import EngineMixin as BaseEngineMixin
from django.forms.renderers import TemplatesSetting as BaseTemplatesSetting
from django.template.backends.django import DjangoTemplates as DjangoTemplatesBackend
from django.utils.functional import cached_property
from django.utils.module_loading import import_string
from django.utils.safestring import mark_safe

from .constants import Action


def render_turbo_stream(
    action: Action,
    target: str,
    content: str = "",
    is_safe: bool = False,
    renderer: Optional[BaseRenderer] = None,
) -> str:
    """Wraps content in correct <turbo-stream> tags.

    :param action: action type
    :param target: the DOM ID target of the stream
    :param content: content to be wrapped. Can be empty.
    :param is_safe: mark content safe for HTML escaping.
    :param renderer: template renderer class

    :return: *<turbo-stream>* string
    """
    if is_safe:
        content = mark_safe(content)

    renderer = renderer or get_default_renderer()

    return (
        renderer.get_template("turbo_response/turbo_stream.html")
        .render({"action": action.value, "target": target, "content": content})
        .strip()
    )


def render_turbo_frame(
    dom_id: str,
    content: str = "",
    is_safe: bool = False,
    renderer: Optional[BaseRenderer] = None,
) -> str:
    """

    Wraps a response in correct *<turbo-frame>* tags.

    :param dom_id: a DOM ID present in the content
    :param content: content of the turbo-frame
    :param is_safe: mark content safe for HTML escaping.
    :param renderer: template renderer class

    :return: *<turbo-frame>* string
    """
    if is_safe:
        content = mark_safe(content)

    renderer = renderer or get_default_renderer()

    return (
        renderer.get_template("turbo_response/turbo_frame.html")
        .render({"dom_id": dom_id, "content": content})
        .strip()
    )


@lru_cache()
def get_default_renderer() -> BaseRenderer:
    """Get the default renderer class.

    :return: renderer subclass: default DjangoTemplates

    """
    from django.conf import settings

    renderer_class_name = getattr(
        settings, "TURBO_RESPONSE_RENDERER", "turbo_response.renderers.DjangoTemplates"
    )

    renderer_class = import_string(renderer_class_name)
    return renderer_class()


class EngineMixin(BaseEngineMixin):
    # uses the django widget renderer resolution to determine backend
    @cached_property
    def engine(self):
        return self.backend(
            {
                "APP_DIRS": True,
                "DIRS": [pathlib.Path(__file__).parent / self.backend.app_dirname],
                "NAME": "turbo_response",
                "OPTIONS": {},
            }
        )


class DjangoTemplates(EngineMixin, BaseRenderer):
    backend = DjangoTemplatesBackend


class Jinja2(EngineMixin, BaseRenderer):
    @cached_property
    def backend(self):
        from django.template.backends.jinja2 import Jinja2

        return Jinja2


class TemplatesSetting(BaseTemplatesSetting):
    ...



# Third Party Libraries
import pytest

# Django Turbo Response
from turbo_response.tests.testapp.models import TodoItem


@pytest.fixture
def todo():
    return TodoItem.objects.create(description="test")



# Standard Library
import http

# Django
from django.http import HttpResponse

# Django Turbo Response
from turbo_response import (
    TurboStream,
    TurboStreamResponse,
    TurboStreamStreamingResponse,
)
from turbo_response.decorators import turbo_stream_response


class TestTurboStreamResponse:
    def test_view_returns_response(self, rf):
        req = rf.get("/")

        @turbo_stream_response
        def my_view(req):
            return HttpResponse("OK")

        resp = my_view(req)
        assert resp["Content-Type"] == "text/html; charset=utf-8"
        assert resp.status_code == http.HTTPStatus.OK

    def test_view_returns_string(self, rf):
        req = rf.get("/")

        @turbo_stream_response
        def my_view(req):
            return TurboStream("messages").replace.render("hello")

        resp = my_view(req)

        assert resp.status_code == http.HTTPStatus.OK
        assert isinstance(resp, TurboStreamResponse)
        assert resp["Content-Type"] == "text/vnd.turbo-stream.html; charset=utf-8"
        assert 'target="messages"' in str(resp.content)

    def test_view_returns_list(self, rf):
        req = rf.get("/")

        @turbo_stream_response
        def my_view(req):
            return [
                TurboStream("messages").replace.render("hello"),
                TurboStream("header").replace.render("world"),
            ]

        resp = my_view(req)

        assert isinstance(resp, TurboStreamResponse)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/vnd.turbo-stream.html; charset=utf-8"
        assert 'target="messages"' in str(resp.content)
        assert 'target="header"' in str(resp.content)

    def test_view_returns_generator(self, rf):
        req = rf.get("/")

        @turbo_stream_response
        def my_view(req):
            yield TurboStream("messages").replace.render("hello")
            yield TurboStream("header").replace.render("world")

        resp = my_view(req)

        assert isinstance(resp, TurboStreamStreamingResponse)
        assert resp.status_code == http.HTTPStatus.OK
        assert resp["Content-Type"] == "text/vnd.turbo-stream.html; charset=utf-8"
        items = list(resp.streaming_content)
        assert len(items) == 2
        assert 'target="messages"' in str(items[0])
        assert 'target="header"' in str(items[1])



# Standard Library
import http

# Django
from django import forms

# Third Party Libraries
import pytest

# Django Turbo Response
from turbo_response.shortcuts import redirect_303, render_form_response

pytestmark = pytest.mark.django_db


class TestRedirect303:
    def test_plain_url(self):
        resp = redirect_303("/")
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        assert resp.url == "/"

    def test_view_name(self):
        resp = redirect_303("index")
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        assert resp.url == "/"

    def test_model(self, todo):
        resp = redirect_303(todo)
        assert resp.status_code == http.HTTPStatus.SEE_OTHER
        assert resp.url == f"/todos/{todo.id}/"


class TestRenderFormResponse:
    class MyForm(forms.Form):
        comment = forms.CharField()

    def test_render_turbo_stream_no_errors(self, rf):
        req = rf.get("/")
        form = self.MyForm()
        resp = render_form_response(
            req,
            form,
            "my_form.html",
            turbo_stream_target="my-form",
            turbo_stream_template="_my_form.html",
        )
        assert resp.status_code == http.HTTPStatus.OK
        assert resp.context_data["form"] == form
        assert resp.context_data["turbo_stream_target"] == "my-form"

    def test_render_turbo_stream_errors(self, rf):
        req = rf.get("/")
        form = self.MyForm({})
        # missing comment, should be error
        assert not form.is_valid()
        assert form.errors
        resp = render_form_response(
            req,
            form,
            "my_form.html",
            turbo_stream_target="my-form",
            turbo_stream_template="_my_form.html",
        )
        assert resp.status_code == http.HTTPStatus.OK
        assert resp.context_data["form"] == form
        assert resp.context_data["turbo_stream_target"] == "my-form"

    def test_render_no_errors(self, rf):
        req = rf.get("/")
        form = self.MyForm()
        resp = render_form_response(req, form, "my_form.html")
        assert resp.status_code == http.HTTPStatus.OK
        assert resp.context_data["form"] == form

    def test_render_errors(self, rf):
        req = rf.get("/")
        form = self.MyForm({})
        # missing comment, should be error
        assert not form.is_valid()
        assert form.errors
        resp = render_form_response(req, form, "my_form.html")
        assert resp.status_code == http.HTTPStatus.UNPROCESSABLE_ENTITY
        assert resp.context_data["form"] == form



# Django Turbo Response
from turbo_response import (
    Action,
    render_turbo_frame_template,
    render_turbo_stream_template,
)


class TestRenderTurboStreamTemplate:
    def test_render(self):
        s = render_turbo_stream_template(
            "simple.html", {"msg": "my content"}, action=Action.UPDATE, target="test"
        ).strip()
        assert (
            s
            == '<turbo-stream action="update" target="test"><template><div>my content</div></template></turbo-stream>'
        )

    def test_render_with_xss(self):
        s = render_turbo_stream_template(
            "simple.html",
            {"msg": "<script></script>"},
            action=Action.UPDATE,
            target="test",
        ).strip()
        assert (
            s
            == '<turbo-stream action="update" target="test"><template><div>&lt;script&gt;&lt;/script&gt;</div></template></turbo-stream>'
        )


class TestRenderTurboTemplate:
    def test_render(self):
        s = render_turbo_frame_template(
            "simple.html", {"msg": "my content"}, dom_id="test"
        )
        assert s == '<turbo-frame id="test"><div>my content</div></turbo-frame>'

    def test_render_with_xss(self):
        s = render_turbo_frame_template(
            "simple.html", {"msg": "<script></script>"}, dom_id="test"
        )
        assert (
            s
            == '<turbo-frame id="test"><div>&lt;script&gt;&lt;/script&gt;</div></turbo-frame>'
        )



# Django
from django.http import HttpResponse

# Third Party Libraries
import pytest

# Django Turbo Response
from turbo_response.middleware import TurboMiddleware


@pytest.fixture
def get_response():
    return lambda req: HttpResponse()


class TestTurboMiddeware:
    def test_accept_header_not_found(self, rf, get_response):
        req = rf.get("/", HTTP_ACCEPT="text/html")
        TurboMiddleware(get_response)(req)
        assert not req.turbo
        assert req.turbo.frame is None

    def test_accept_header_found(self, rf, get_response):
        req = rf.get("/", HTTP_ACCEPT="text/vnd.turbo-stream.html")
        TurboMiddleware(get_response)(req)
        assert req.turbo
        assert req.turbo.frame is None

    def test_turbo_frame(self, rf, get_response):
        req = rf.get(
            "/",
            HTTP_ACCEPT="text/vnd.turbo-stream.html",
            HTTP_TURBO_FRAME="my-playlist",
        )
        TurboMiddleware(get_response)(req)
        assert req.turbo
        assert req.turbo.frame == "my-playlist"
